/// @file openLayers4.js
/// @author Mario Chirinos Colunga
/// @date 2016-04-14
/// 

//------------------------------------------------------------------------------
/**
 * Definitions for the languages.
 * @param  divID div name to insert the map
 * @param  lng Longitude
 * @param  lat Latitude
 * @param  z Zoom
 * @return OpenStreetMapsClass object
 */
function OpenStreetMapsClass(divID, lng, lat, z)
{
	//this.vectorLayers = [];
	var self=this;
	this.vectorLayer = new ol.layer.Vector({source: new ol.source.Vector({features: []})});
	this.map = new ol.Map({
				layers: [new ol.layer.Tile({
							//source: new ol.source.OSM()
							source: new ol.source.XYZ({url:'https://api.tiles.mapbox.com/v4/mapbox.light/{z}/{x}/{y}.png?access_token=pk.eyJ1IjoibWFwYm94IiwiYSI6ImNpejY4NXVycTA2emYycXBndHRqcmZ3N3gifQ.rJcFIG214AriISLbB6B5aw'})

							}),this.vectorLayer],
				target: 'map',
				controls: ol.control.defaults({attributionOptions: ({collapsible: true})}),
				view: new ol.View({ center: ol.proj.fromLonLat([lng,lat]),  zoom: z})
				});

	this.map.on('click', function(e) {self.onMapClick(e)});

	this.map.on("pointermove", function (e)
	{
		var hit = this.forEachFeatureAtPixel(e.pixel, function(feature, layer) {return true;}); 
		if (hit)
			this.getTargetElement().style.cursor = 'pointer';
		else
			this.getTargetElement().style.cursor = '';
	});
};
//------------------------------------------------------------------------------
OpenStreetMapsClass.prototype.onMapClick = function(e)
{
	console.log("onMapClick");
	var feature = this.map.forEachFeatureAtPixel(e.pixel,function(feature) {return feature;});
	if(feature && feature.onClick!==undefined)
		feature.onClick(e);
}
//------------------------------------------------------------------------------

/**
 *	Browser Geolocation
 */
OpenStreetMapsClass.prototype.geolocation = function()
{
	var self = this;
	var geoOptions = {
				enableHighAccuracy: true,
				timeout: 5000,
				maximumAge: 0
			};
	if (navigator.geolocation)
	{
		navigator.geolocation.getCurrentPosition(function(position){
									console.log("navigator geolocation:"+position.coords.longitude +","+ position.coords.latitude);
									self.map.getView().setCenter( ol.proj.fromLonLat([position.coords.longitude, position.coords.latitude]));
											

									},
							function(error) {
									console.log("ERROR "+error.code+": navigator geolocation");

									},

								geoOptions);
	}
	else
	{
		console.log("NO navigator geolocation");
	}
}
//------------------------------------------------------------------------------
/**
 *	Draw a polygon in the map
 *	@param coords coords to draw
 */
OpenStreetMapsClass.prototype.addPolygon = function(coords)
{
	// create feature
	var feature = new ol.Feature({
	    geometry: new ol.geom.Polygon(coords)
	})

	// fit zoom with animation
	this.map.getView().fit(feature.getGeometry(), {duration: 1000, padding: [0, 0, 70, 0]});

	// add feature to map
	this.vectorLayer.getSource().addFeature(feature);

	// show coords
	this.showCoords(feature.getGeometry());
}
//------------------------------------------------------------------------------
/**
 *	Draw a polygon in the map
 *	@param wkt coords int wkt format to draw
 */
OpenStreetMapsClass.prototype.addWKTPolygon = function(wkt)
{
	var format = new ol.format.WKT();

	var feature = format.readFeature(wkt, {
		dataProjection: 'EPSG:4326',
		featureProjection: 'EPSG:3857'
	});

	this.map.getView().fit(feature.getGeometry(), {duration: 1000, padding: [0, 0, 70, 0]});
	this.vectorLayer.getSource().addFeature(feature);

	document.getElementById("id_polygon").value = wkt;
}

OpenStreetMapsClass.prototype.deletefootprint = function (prevFeature) {
	if (prevFeature){
		this.vectorLayer.getSource().removeFeature(prevFeature);
	}
}

OpenStreetMapsClass.prototype.addfootprint = function(wkt, prevFeature){
	var format = new ol.format.WKT();

	var feature = format.readFeature(wkt, {
		dataProjection: 'EPSG:4326',
		featureProjection: 'EPSG:3857'
	});

    var style = new ol.style.Style({
		stroke: new ol.style.Stroke({
			color: getRandomColor(),
			width: 1
		}),
		fill: new ol.style.Fill({
			color: 'rgba(255,255,0,0.1)'
		})
	})

	feature.setStyle(style);
	
	this.map.getView().fit(feature.getGeometry(), {duration: 1000, padding: [0, 0, 70, 0]});
	this.vectorLayer.getSource().addFeature(feature);

	return feature;
}

//------------------------------------------------------------------------------
/**
 *	calculate the area of each polygon and return the coords of the biggest polygon
 *	@param coords polygons coords
 *	@return biggest coords of the biggest polygon
 */
OpenStreetMapsClass.prototype.getBiggestPolygon = function(coords)
{
	if (coords.length <= 1) { return coords; }

	// calculate areas
	var areas = [];
	for (var index in coords) {
		var polygon = new ol.geom.Polygon([coords[index]]);
		var area = ol.Sphere.getArea(polygon);
		areas.push(area);
	}

	// get coords with max area
	var max = Math.max.apply(null, areas);
	var biggest = coords[areas.indexOf(max)];

	return [biggest];
}
//------------------------------------------------------------------------------
/**
 *	format database coords to openlayers coords
 *	@param coords database coords array
 *	@return formatCoords coords with correct format
 */
OpenStreetMapsClass.prototype.formatCoords = function(coords)
{
	var formatCoords = [];

	// algunos municipios tienes regiones separadas (campeche tiene 4, merida tiene 1)
	for (var index = coords.length - 1; index >= 0; index--) {
		var regionCoords = [];
		var region = coords[index][0];

		// se formatea las coordenadas de una region
		for (var regionIndex = region.length - 1; regionIndex >= 0; regionIndex--) {
			var c = region[regionIndex];
			regionCoords.push(ol.proj.fromLonLat([parseFloat(c[0]), parseFloat(c[1])], 'EPSG:3857', 'EPSG:4326'));
		}

		formatCoords.push(regionCoords);
	}

	return formatCoords;
}
//------------------------------------------------------------------------------
/**
 *	show values of a polygon with wkt format
 * 	POLYGON ((30 10, 40 40, 20 40, 10 20, 30 10))
 *	MULTIPOLYGON (((40 40, 20 45, 45 30, 40 40)),((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20)))
 *	@param geometry the geometry to show e.g. a polygon
 */
OpenStreetMapsClass.prototype.showCoords = function(geometry)
{
	var wkt = new ol.format.WKT();
	var wkt_polygon = wkt.writeGeometry(geometry.clone().transform( 'EPSG:3857', 'EPSG:4326'));
	document.getElementById("id_polygon").value = wkt_polygon;
}
//------------------------------------------------------------------------------
/**
 *	delete current polygon in map
 */
OpenStreetMapsClass.prototype.removePolygon = function()
{
	this.vectorLayer.getSource().clear();
}
//------------------------------------------------------------------------------
/**
 *	@param lng Longitude
 *	@param lat Latitude
 *	@pram style ol.style.Style
 *	@return feature added
 */
OpenStreetMapsClass.prototype.addFeatureToVectorLayer = function(lng,lat,style,jsonObj)
{

	var iconFeature = new ol.Feature({
					geometry: new ol.geom.Point(ol.proj.fromLonLat([lng,lat])), json: jsonObj
				});
   	iconFeature.setStyle(style);
	//iconFeature.on("click", function(e){console.log("click")}, this);

	this.vectorLayer.getSource().addFeature(iconFeature);

	return iconFeature;

}
//------------------------------------------------------------------------------
/**
 *	@param	url icon file location
 *	@param	x offset X [0..1] relative to icon width
 *	@param	y offset Y [0..1] relative to icon height
 */
OpenStreetMapsClass.prototype.iconStyle = function(url,x,y)
{
	return new ol.style.Style({
		image: new ol.style.Icon( ({
		  anchor: [x, y],
		  anchorXUnits: 'fraction',
		  anchorYUnits: 'fraction',
		  src: url
		}))
	      });

}

function getRandomColor() {
	var letters = '0123456789ABCDEF';
	var color = '#';
	for (var i = 0; i < 6; i++) {
		color += letters[Math.floor(Math.random() * 16)];
	}
	return color;
}